#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/io.h>

#include "uart.h"
#include "dcf77/dcf77.h"

char daytime[] = "\r  :  :     .  .20  \x0";
//                01234567890123456789012

char *
get_time_str(char* timestr)
{
  timestr[2] = (time.hour % 10) + '0';
  timestr[1] = (time.hour / 10) + '0';
  timestr[5] = (time.minute % 10) + '0';
  timestr[4] = (time.minute / 10) + '0';
  timestr[8] = (time.second % 10) + '0';
  timestr[7] = (time.second / 10) + '0';

  timestr[11] = (time.day % 10) + '0';
  timestr[10] = (time.day / 10) + '0';
  timestr[14] = (time.month % 10) + '0';
  timestr[13] = (time.month / 10) + '0';
  timestr[19] = (time.year % 10) + '0';
  timestr[18] = (time.year / 10) + '0';
  if (!synchronize)
    timestr[20] = '*'; // not in sync
  else
    timestr[20] = ' '; // in sync
}


//*****************************************************************************************************************
int
main(void)
{
  uint8_t flipper;
  // Uart initialisierung zu Ausgabe von Daten
  uinit();
  timebase_init();
  flipper = 0;

  sei();
  uputs((uint8_t*) "\r\n\n\n\nDCF77 by Peter Danneger\n\n\r");

  DDRD |= 3; // my test-pins
  DDRL = 0xff;
  while (1)
    {
      scan_dcf77();     // has to be called at least once every 100ms
#if 1
      if (DCF77_PIN & 1 << DCF77)
        PORTD &= ~(1 << PD0); // PD0 shows dcf77-pulse
      else
        PORTD |= 1 << PD0;
#endif
      if (timeflags & 1 << ONE_SECOND)
        {
          timeflags = 0;
          clock();
          PORTL=synchronize;
          if (!synchronize)
            {
            PORTD &= ~(1 << PD1); // in sync
              switch (flipper++)
                {
              case 0:
                uputs((uint8_t *)"Wait for Sync -    \r");
                break;

              case 1:
                uputs((uint8_t *)"Wait for Sync \\    \r");
              break;

              case 2:
                uputs((uint8_t *)"Wait for Sync |    \r");
                break;

              case 3:
                uputs((uint8_t *)"Wait for Sync /    \r");
                break;

                }
              if(flipper == 4)
                flipper=0;
              // no valid Time
            }
          else // active-low LED on PD1 shows sync-state
            {
              PORTL=synchronize;
              get_time_str((char *) daytime);
              uputs((uint8_t *) daytime);
              PORTD |= (1 << PD1); // out of sync
              // now we can use time (struct) safely
            }
        }

    }
  return 0;
}
